package ru.dbotthepony.mc.otm;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.TickEvent;
import net.minecraftforge.event.server.ServerAboutToStartEvent;
import net.minecraftforge.event.server.ServerStoppingEvent;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.InterModComms;
import net.minecraftforge.fml.LogicalSide;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.fml.event.lifecycle.InterModEnqueueEvent;
import net.minecraftforge.fml.event.lifecycle.InterModProcessEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityBlackHole;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.drive.DrivePool;
import ru.dbotthepony.mc.otm.client.AndroidGui;
import ru.dbotthepony.mc.otm.client.EventHandler;
import ru.dbotthepony.mc.otm.core.Fraction;
import ru.dbotthepony.mc.otm.item.ItemPortableCondensationDrive;
import ru.dbotthepony.mc.otm.matter.MatterRegistry;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper;
import ru.dbotthepony.mc.otm.storage.StorageObjectRegistry;

import javax.annotation.ParametersAreNonnullByDefault;
import java.util.ArrayList;
import java.util.WeakHashMap;
import java.util.function.Consumer;
import java.util.function.Supplier;
import java.util.stream.Collectors;

// The value here should match an entry in the META-INF/mods.toml file
@Mod(OverdriveThatMatters.MOD_ID)
@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class OverdriveThatMatters {
	// Directly reference a log4j logger.
	public static final String MOD_ID = "overdrive_that_matters";
	public static final Logger LOGGER = LogManager.getLogger();

	public static AndroidGui ANDROID_GUI;

	public static CreativeModeTab CREATIVE_TAB;

	// why no LinkedList?
	// because you can't get nodes directly, which is vital in getting this thing working faster.
	private static final WeakHashMap<Level, ArrayList<Supplier<Boolean>>> tick_until = new WeakHashMap<>();
	private static final WeakHashMap<Level, ArrayList<Consumer<Level>>> tick_once = new WeakHashMap<>();

	@SubscribeEvent(priority = EventPriority.HIGHEST)
	@SuppressWarnings("unused")
	public void onServerStarting(ServerAboutToStartEvent event) {
		tick_until.clear();
	}

	@SubscribeEvent(priority = EventPriority.HIGHEST)
	@SuppressWarnings("unused")
	public void onServerStopping(ServerStoppingEvent event) {
		tick_until.clear();
	}

	@SubscribeEvent(priority = EventPriority.LOWEST)
	@SuppressWarnings("unused")
	public void onPreTick(TickEvent.WorldTickEvent event) {
		if (event.phase != TickEvent.Phase.START || event.side != LogicalSide.SERVER)
			return;

		final var until = tick_until.get(event.world);

		if (until != null) {
			for (int i = until.size() - 1; i >= 0; i--) {
				if (until.get(i).get()) {
					until.remove(i);
				}
			}

			if (until.size() == 0) {
				tick_until.remove(event.world);
			}
		}

		final var once = tick_once.get(event.world);

		if (once != null) {
			ArrayList<Supplier<Boolean>> invalid = new ArrayList<>();

			for (var ticker : once) {
				ticker.accept(event.world);
			}

			tick_once.remove(event.world);
		}
	}

	public static void tickUntil(Level level, Supplier<Boolean> ticker) {
		tick_until.computeIfAbsent(level, (k) -> new ArrayList<>()).add(ticker);
	}

	public static void tickOnceSelf(Level level, Consumer<Level> ticker) {
		tick_once.computeIfAbsent(level, (k) -> new ArrayList<>()).add(ticker);
	}

	public static void tickOnce(Level level, Runnable ticker) {
		tick_once.computeIfAbsent(level, (k) -> new ArrayList<>()).add((l) -> ticker.run());
	}

	public OverdriveThatMatters() {
		// Register the setup method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setup);
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::setupClient);
		// Register the enqueueIMC method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::enqueueIMC);
		// Register the processIMC method for modloading
		FMLJavaModLoadingContext.get().getModEventBus().addListener(this::processIMC);

		// Register ourselves for server and other game events we are interested in
		MinecraftForge.EVENT_BUS.register(this);
		MinecraftForge.EVENT_BUS.register(AndroidCapabilityPlayer.Companion);
		MinecraftForge.EVENT_BUS.register(AndroidCapability.Companion);
		MinecraftForge.EVENT_BUS.register(MatterRegistry.class);
		MinecraftForge.EVENT_BUS.register(BlockEntityBlackHole.BlackHoleExplosionQueue.class);

		FMLJavaModLoadingContext.get().getModEventBus().register(MatteryCapability.class);

		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Items.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Blocks.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.BlockEntities.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Menus.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.AndroidFeatures.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.AndroidResearch.class);
		FMLJavaModLoadingContext.get().getModEventBus().register(Registry.Stats.class);

		MinecraftForge.EVENT_BUS.register(DrivePool.INSTANCE);
		MinecraftForge.EVENT_BUS.register(ItemPortableCondensationDrive.Companion);

		FMLJavaModLoadingContext.get().getModEventBus().addListener(AndroidCapability::registerEffects);

		// LOGGER.info("Registered event handlers");

		// force Registry static initializer to be called
		Registry.dummy();

		CREATIVE_TAB = new CreativeModeTab("otm") {
			@Override
			public ItemStack makeIcon() {
				return new ItemStack(Registry.Items.BATTERY_CREATIVE, 1);
			}
		};
	}

	private void setup(final FMLCommonSetupEvent event) {
		MatteryNetworking.register();
		// LOGGER.info("Registered network");

		MatterRegistry.registerInitialItems();

		StorageObjectRegistry.register(ItemStackWrapper.class, ItemStackWrapper.EMPTY, new Fraction("3.125"));
	}

	private void setupClient(final FMLClientSetupEvent event) {
		ANDROID_GUI = new AndroidGui();
		MinecraftForge.EVENT_BUS.register(ANDROID_GUI);
		MinecraftForge.EVENT_BUS.register(AndroidGui.class);
		MinecraftForge.EVENT_BUS.register(EventHandler.class);
	}

	private void enqueueIMC(final InterModEnqueueEvent event) {
		// some example code to dispatch IMC to another mod
		InterModComms.sendTo("examplemod", "helloworld", () -> { LOGGER.info("Hello world from the MDK"); return "Hello world";});
	}

	private void processIMC(final InterModProcessEvent event) {
		// some example code to receive and process InterModComms from other mods
		LOGGER.info("Got IMC {}", event.getIMCStream().
				map(m->m.messageSupplier().get()).
				collect(Collectors.toList()));
	}
}
