package ru.dbotthepony.mc.otm.capability.drive

import net.minecraft.nbt.CompoundTag
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraftforge.registries.RegistryManager
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.set
import ru.dbotthepony.mc.otm.storage.IStorageTuple
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper
import ru.dbotthepony.mc.otm.storage.StorageObjectRegistry
import ru.dbotthepony.mc.otm.storage.StorageObjectTuple
import java.util.*

class ItemMatteryDrive : AbstractMatteryDrive<ItemStackWrapper>, IItemMatteryDrive {
	constructor(capacity: Fraction, max_different_stacks: Int) : super(capacity, maxDifferentStacks = max_different_stacks)
	constructor(capacity: Fraction, uuid: UUID, max_different_stacks: Int) : super(capacity, uuid, max_different_stacks)
	constructor(capacity: Fraction, uuid: UUID) : super(capacity, uuid)
	constructor(capacity: Fraction) : super(capacity)

	private val identity = StorageObjectRegistry.getOrError(ItemStackWrapper::class.java)
	override fun identity() = identity

	fun insertObject(item: ItemStack, simulate: Boolean): ItemStack {
		return insertStack(ItemStackWrapper(item), simulate).stack
	}

	override fun serializeStack(item: IStorageTuple<ItemStackWrapper>): CompoundTag? {
		val tag = CompoundTag()
		val location = item.stack().stack.item.registryName!!.toString()

		tag["item"] = location
		tag["count"] = item.stack.stack.count

		val itag = item.stack.stack.tag

		if (itag != null) {
			tag["data"] = itag
		}

		return tag
	}

	override fun deserializeStack(tag: CompoundTag): ItemStackWrapper? {
		val item = RegistryManager.ACTIVE.getRegistry(Item::class.java).getValue(ResourceLocation(tag.getString("item")))

		if (item != null && item !== Items.AIR) {
			val count = tag.getInt("count")
			val itemstack = ItemStack(item, count)
			itemstack.tag = tag["data"] as? CompoundTag?
			return ItemStackWrapper(itemstack)
		}

		return null
	}

	override fun findItems(item: Item): List<IStorageTuple<ItemStackWrapper>> {
		val list = storedStacks[item]
		return if (list != null) java.util.List.copyOf(list) else emptyList()
	}

	override fun findItems(stack: ItemStack): List<IStorageTuple<ItemStackWrapper>> {
		val list = storedStacks[stack.item] ?: return emptyList()

		var amount = 0

		for (_stack in list) {
			if (ItemStack.tagMatches(_stack.stack.stack, stack)) {
				amount++
			}
		}

		val build_list = ArrayList<IStorageTuple<ItemStackWrapper>>(amount)
		var i = 0

		for (_stack in list) {
			if (ItemStack.tagMatches(_stack.stack.stack, stack)) {
				build_list[i] = _stack
				i++
			}
		}

		return build_list
	}

	companion object {
		@JvmField
		val DUMMY = ItemMatteryDrive(Fraction(0), UUID(0L, 0L), 0)
	}
}