package ru.dbotthepony.mc.otm.client.screen

import net.minecraft.network.chat.Component
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.client.screen.panels.*
import ru.dbotthepony.mc.otm.client.screen.widget.PowerGaugePanel
import ru.dbotthepony.mc.otm.menu.ItemMonitorMenu

class ItemMonitorScreen(menu: ItemMonitorMenu, inventory: Inventory, title: Component) :
	MatteryScreen<ItemMonitorMenu>(menu, inventory, title) {

	override fun makeMainFrame(): FramePanel {
		val frame = FramePanel(this, null, 0f, 0f, FRAME_WIDTH, FRAME_HEIGHT, getTitle())

		PowerGaugePanel(this, frame, menu.battery_widget, 8f, 29f)
		SlotPanel(this, frame, menu.battery_slot, 8f, 80f)

		val scrollBar = ScrollBarPanel(this, frame, 192f, 14f, 92f)
		scrollBar.setupRowMultiplier { menu.view.items.size / GRID_WIDTH }

		val gridPanel = GridPanel(this, frame, 28f, 16f, GRID_WIDTH * 18f, GRID_HEIGHT * 18f, GRID_WIDTH, GRID_HEIGHT)

		for (i in 0 until GRID_WIDTH * GRID_HEIGHT) {
			object : AbstractSlotPanel(this@ItemMonitorScreen, gridPanel) {
				override fun getItemStack(): ItemStack {
					val index = i + scrollBar.getScroll(menu.view.items.size / GRID_WIDTH)
					val list = menu.view.items
					return if (index >= list.size) ItemStack.EMPTY else list[index].stack()
				}

				override fun mouseScrolledInner(mouse_x: Double, mouse_y: Double, scroll: Double): Boolean {
					return scrollBar.mouseScrolledInner(mouse_x, mouse_y, scroll)
				}

				override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, mouse_click_type: Int): Boolean {
					val index = i + scrollBar.getScroll(GRID_WIDTH)
					menu.view.mouseClick(index, mouse_click_type)
					return true
				}
			}
		}

		return frame
	}

	companion object {
		const val FRAME_WIDTH = 210f
		const val FRAME_HEIGHT = 110f
		const val GRID_WIDTH = 9
		const val GRID_HEIGHT = 5
	}
}